﻿#region Header
/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#endregion

#region Using
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.DirectX.AudioVideoPlayback;
using System.IO; 
#endregion

namespace Little_Alarm_Clock
{
    public partial class AlarmFrm : Form
    {
        public delegate void SnoozeTimer(object sender, SnoozeEventArgs e);
        public event SnoozeTimer AddSnoozeTimer;
        private Audio objAudio;

        public AlarmFrm(DateTime dateTime, string strDescription)
        {
            InitializeComponent();

            this.Icon = SystemIcons.Warning;
            this.textBox1.Text = string.Format("It is now {0:T}\r\nDescription: {1}", dateTime, strDescription);

            PlayAlarm();
        }

        void PlayAlarm()
        {
            string strAlarmPath = Properties.Settings.Default.strAlarmPath;
            bool blnPlaySound = Properties.Settings.Default.bPlaySound;

            if (blnPlaySound)
            {
                FileInfo fiFile = new FileInfo(strAlarmPath);
                if (fiFile.Exists)
                {
#if Release
                this.objAudio = new Audio(strAlarmPath);
                this.objAudio.Play();

                if (Properties.Settings.Default.bRepeatSound)
                    this.objAudio.Ending += new EventHandler(objAudio_Ending);
#endif
                }
                else
                {
                    MessageBox.Show("Audio file not found: " + strAlarmPath, "File not found",
                       MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //No sound
            }
        }

        void objAudio_Ending(object sender, EventArgs e)
        {
            PlayAlarm();
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            if (this.objAudio != null)
                this.objAudio.Stop();

            this.Close();
        }

        private void buttonSnooze_Click(object sender, EventArgs e)
        {
            // Display alarm 9 minutes from now
            SnoozeEventArgs args = new SnoozeEventArgs(DateTime.Now.AddMinutes(9));

            AddSnoozeTimer(this, args);
        }

        public class SnoozeEventArgs : EventArgs
        {
            private DateTime _dateTime;

            public SnoozeEventArgs(DateTime dateTime)
            {
                if (dateTime != null)
                    this._dateTime = dateTime;
            }

            public DateTime dateTime
            {
                get { return _dateTime; }
            }
        }
    }
}
