﻿/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Little_Alarm_Clock
{
    public partial class CreateFrm : Form
    {
        // add a delegate
        public delegate void AddNewAlarm(object sender, AlarmEventArgs e);

        // add event for delegate type
        public event AddNewAlarm NewAlarmAdded;

        public CreateFrm()
        {
            InitializeComponent();

            // Set default values
            this.comboBoxTime.Text = this.comboBoxTime.Items[0].ToString();
            this.comboBoxRepeat.Text = this.comboBoxRepeat.Items[0].ToString();
            this.dateTimePicker.Value = DateTime.Now;
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            DateTime dateTime = DateTime.Now;

            if (string.IsNullOrEmpty(this.textBoxDesc.Text))
            {
                MessageBox.Show(this, "You must provide a description", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (this.comboBoxTime.Text != null)
            {
                if (this.comboBoxTime.Text.CompareTo("Manual") == 0)
                    dateTime = dateTimePicker.Value;
                else if (this.comboBoxTime.Text.CompareTo("10 Min Later") == 0)
                    dateTime = DateTime.Now.AddMinutes(10);
                else if (this.comboBoxTime.Text.CompareTo("20 Min Later") == 0)
                    dateTime = DateTime.Now.AddMinutes(20);
                else if (this.comboBoxTime.Text.CompareTo("30 Min Later") == 0)
                    dateTime = DateTime.Now.AddMinutes(30);
                else if (this.comboBoxTime.Text.CompareTo("45 Min Later") == 0)
                    dateTime = DateTime.Now.AddMinutes(45);
                else if (this.comboBoxTime.Text.CompareTo("1 Hour Later") == 0)
                    dateTime = DateTime.Now.AddHours(1);
            }

            AlarmEventArgs args = new AlarmEventArgs(dateTime, this.comboBoxRepeat.Text, this.textBoxDesc.Text);
            NewAlarmAdded(this, args);
            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void comboBoxTime_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.comboBoxTime.Text.CompareTo("Manual") == 0)
                this.dateTimePicker.Enabled = true;
            else
                this.dateTimePicker.Enabled = false;

            if (this.comboBoxTime.Text.CompareTo("10 Min Later") == 0)
                this.dateTimePicker.Value = DateTime.Now.AddMinutes(10);
            else if (this.comboBoxTime.Text.CompareTo("20 Min Later") == 0)
                this.dateTimePicker.Value = DateTime.Now.AddMinutes(20);
            else if (this.comboBoxTime.Text.CompareTo("30 Min Later") == 0)
                this.dateTimePicker.Value = DateTime.Now.AddMinutes(30);
            else if (this.comboBoxTime.Text.CompareTo("45 Min Later") == 0)
                this.dateTimePicker.Value = DateTime.Now.AddMinutes(45);
            else if (this.comboBoxTime.Text.CompareTo("1 Hour Later") == 0)
                this.dateTimePicker.Value = DateTime.Now.AddHours(1);
        }


    }

    public class AlarmEventArgs : EventArgs
    {
        private DateTime _dateTime;
        private string _strRepeat;
        private string _strDescription;
        private int _nListViewIndex;

        /// <summary>
        /// Parameters used to add an alarm
        /// </summary>
        public AlarmEventArgs(DateTime dateTime, string strRepeat, string strDescription)
        {
            if (dateTime != null)
                this._dateTime = dateTime;

            if (strRepeat != null)
                this._strRepeat = strRepeat;

            if (strDescription != null)
                this._strDescription = strDescription;
        }

        /// <summary>
        /// Parameters used to edit alarms
        /// </summary>
        public AlarmEventArgs(DateTime dateTime, string strRepeat, string strDescription, int nListViewIndex)
        {
            if (dateTime != null)
                this._dateTime = dateTime;

            if (strRepeat != null)
                this._strRepeat = strRepeat;

            if (strDescription != null)
                this._strDescription = strDescription;

            if (nListViewIndex.ToString() != null)
                this._nListViewIndex = nListViewIndex;
        }

        public DateTime dateTime
        {
            get { return _dateTime; }
        }

        public string strRepeat
        {
            get { return _strRepeat; }
        }

        public string strDescription
        {
            get { return _strDescription; }
        }

        public int nListViewIndex
        {
            get { return _nListViewIndex; }
        }
    }
}
