﻿#region Header
/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#endregion

#region Using
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using Little_Alarm_Clock.Logic; 
#endregion

namespace Little_Alarm_Clock
{
    #region struct structAlarms
    /// <summary>
    /// Alarm structure used for settings
    /// </summary>
    [Serializable]
    public struct structAlarms
    {
        public DateTime dateTime;
        public string strRepeats;
        public string strDescription;
    } 
    #endregion

    public partial class MainFrm : Form
    {
        #region Member
        private AlarmFrm alarmDlg;
        private ProgramUpdater m_puUpdater = null;
        private UpdateFrm m_dlgUpdate = null;
        AboutBox m_aboutFrm = null;
        #endregion

        #region Ctor
        public MainFrm()
        {
            InitializeComponent();

            m_puUpdater = new ProgramUpdater();
            m_puUpdater.Init();

            m_dlgUpdate = new UpdateFrm(m_puUpdater);
            m_aboutFrm = new AboutBox();
        } 
        #endregion

        #region Events
        #region timerTime_Tick
        private void timerTime_Tick(object sender, EventArgs e)
        {
            DateTime dateTimeNow = DateTime.Now;

            labelClock.Text = string.Format("{0:T}", dateTimeNow);

            if (alarmDlg == null)
            {
                for (int i = 0; i < this.listView1.Items.Count; i++)
                {
                    DateTime dateTime = Convert.ToDateTime(this.listView1.Items[i].SubItems[0].Text);
                    string strRepeat = this.listView1.Items[i].SubItems[1].Text;
                    bool bDeleteAlarm = false;

                    // Compare the TimeSpan to current time
                    if (dateTimeNow.TimeOfDay.Hours == dateTime.TimeOfDay.Hours &&
                        dateTimeNow.TimeOfDay.Minutes == dateTime.TimeOfDay.Minutes &&
                        dateTimeNow.TimeOfDay.Seconds == dateTime.TimeOfDay.Seconds)
                    {
                        // See if its right day of a week
                        if (string.Compare(strRepeat, "Once") == 0)
                            bDeleteAlarm = true;
                        else if (string.Compare(strRepeat, "Mon-Fri") == 0)
                        {
                            if (dateTimeNow.DayOfWeek == DayOfWeek.Saturday ||
                                dateTimeNow.DayOfWeek == DayOfWeek.Sunday)
                                return;
                        }
                        else if (string.Compare(strRepeat, "Sat-Sun") == 0)
                        {
                            if (dateTimeNow.DayOfWeek != DayOfWeek.Saturday &&
                                dateTimeNow.DayOfWeek != DayOfWeek.Sunday)
                                return;
                        }

                        // Show balloon tip
                        this.notifyIcon1.BalloonTipTitle = Application.ProductName;
                        this.notifyIcon1.BalloonTipText = string.Format("It is now {0}", DateTime.Now.ToString("hh:mm tt"));
                        this.notifyIcon1.BalloonTipIcon = ToolTipIcon.Info;
                        this.notifyIcon1.ShowBalloonTip(5000);

                        // Display alarm
                        this.alarmDlg = new AlarmFrm(dateTimeNow, this.listView1.Items[i].SubItems[2].Text);
                        this.alarmDlg.AddSnoozeTimer += new AlarmFrm.SnoozeTimer(alarmDlg_AddSnoozeTimer);
                        this.alarmDlg.ShowDialog();
                        this.alarmDlg = null;

                        if (bDeleteAlarm)
                            this.listView1.Items[i].Remove();
                    }
                }
            }
        } 
        #endregion

        #region alarmDlg_AddSnoozeTimer
        void alarmDlg_AddSnoozeTimer(object sender, AlarmFrm.SnoozeEventArgs e)
        {
            ListViewItem listViewItem = new ListViewItem(new string[] { e.dateTime.ToString("hh:mm tt"), "Once", "" });
            this.listView1.Items.Add(listViewItem);
            this.listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        } 
        #endregion

        #region CreateDlg_NewAlarmAdded
        void CreateDlg_NewAlarmAdded(object sender, AlarmEventArgs e)
        {
            ListViewItem listViewItem = new ListViewItem(new string[] { e.dateTime.ToString("hh:mm tt"), e.strRepeat, e.strDescription });
            this.listView1.Items.Add(listViewItem);
            this.listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        } 
        #endregion

        #region MainFrm_FormClosing
        void MainFrm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
            {
                e.Cancel = AskIfClose();
            }

            // Clear old list
            Properties.Settings.Default.arrayAlarms = new ArrayList();

            // Store new entries
            for (int i = 0; i < this.listView1.Items.Count; i++)
            {
                structAlarms structAlarm = new structAlarms();

                structAlarm.dateTime = Convert.ToDateTime(this.listView1.Items[i].SubItems[0].Text);
                structAlarm.strRepeats = this.listView1.Items[i].SubItems[1].Text;
                structAlarm.strDescription = this.listView1.Items[i].SubItems[2].Text;

                Properties.Settings.Default.arrayAlarms.Add(structAlarm);
            }
        }
        #endregion

        #region MainFrm_Shown
        private void MainFrm_Shown(object sender, EventArgs e)
        {
            foreach (string strArgs in Environment.GetCommandLineArgs())
            {
                if (strArgs.CompareTo("/startup") == 0)
                {
                    this.WindowState = FormWindowState.Minimized;
                    this.Hide();
                }
            }

            //Read alarms
            if (Properties.Settings.Default.arrayAlarms != null)
            {
                ArrayList arrayAlarms = Properties.Settings.Default.arrayAlarms;

                for (int i = 0; i < arrayAlarms.Count; i++)
                {
                    structAlarms structAlarm = (structAlarms)arrayAlarms[i];
                    this.listView1.Items.Add(new ListViewItem(new string[] { structAlarm.dateTime.ToString("hh:mm tt"), structAlarm.strRepeats, structAlarm.strDescription }));
                }
            }

            this.timerTime.Start();

            if ((Properties.Settings.Default.bCheckUpdatesAtStart) &&
                (m_puUpdater.UpdateStatus == ProgramUpdateStatus.NewVersion))
            {
                m_dlgUpdate.ShowDialog(this);
            }
            else
            {
                //Nothing
            }
        }
        #endregion

        #region MainFrm_Resize
        void MainFrm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
                this.Hide();
            if (this.listView1.Items.Count > 0)
                this.listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        }
        #endregion

        #region editFrm_AlarmEdited
        void editFrm_AlarmEdited(object sender, AlarmEventArgs e)
        {
            try
            {
                this.listView1.Items[e.nListViewIndex].SubItems[0].Text = e.dateTime.ToString("hh:mm tt");
                this.listView1.Items[e.nListViewIndex].SubItems[1].Text = e.strRepeat;
                this.listView1.Items[e.nListViewIndex].SubItems[2].Text = e.strDescription;
            }
            catch (ArgumentException)
            {
                // Alarm already went off, so re-add the item
                this.listView1.Items.Add(new ListViewItem(new string[] { e.dateTime.ToString("hh:mm tt"), e.strRepeat, e.strDescription }));
            }

            this.listView1.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        } 
        #endregion

        #region clearToolStripMenuItem_Click
        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearAlarms();
        }

        private void ClearAlarms()
        {
            if (MessageBox.Show(this, "Are you sure?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                this.listView1.Items.Clear();
                Properties.Settings.Default.arrayAlarms.Clear();
            }
        } 
        #endregion

        #region notifyIcon1_MouseDoubleClick
        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Show();
                this.WindowState = FormWindowState.Normal;
            }
            else
            {
                this.WindowState = FormWindowState.Minimized;
                this.Hide();
            }
        } 
        #endregion

        #region hideShowLittleAlarmClockToolStripMenuItem_Click
        private void hideShowLittleAlarmClockToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal)
            {
                this.Hide();
                this.WindowState = FormWindowState.Minimized;
            }
            else
            {
                this.Show();
                this.WindowState = FormWindowState.Normal;
            }
        } 
        #endregion

        #region aboutToolStripMenuItem1_Click
        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal)
            {
                AboutBox aboutFrm = new AboutBox();
                aboutFrm.ShowDialog(this);
            }
        } 
        #endregion

        #region exportToolStripMenuItem_Click
        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.listView1.Items.Count <= 0)
            {
                MessageBox.Show(this, "You need to create an alarm first", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            SaveFileDialog saveFileDlg = new SaveFileDialog();

            saveFileDlg.Filter = "XML File (*.xml)|*.xml";

            if (saveFileDlg.ShowDialog(this) == DialogResult.OK)
            {
                XmlWriter xmlWriter = XmlTextWriter.Create(saveFileDlg.OpenFile());

                xmlWriter.WriteStartDocument();
                xmlWriter.WriteStartElement("alarms");

                for (int i = 0; i < this.listView1.Items.Count; i++)
                {
                    xmlWriter.WriteStartElement("alarm");
                    xmlWriter.WriteAttributeString("time", this.listView1.Items[i].SubItems[0].Text);
                    xmlWriter.WriteAttributeString("repeats", this.listView1.Items[i].SubItems[1].Text);
                    xmlWriter.WriteAttributeString("description", this.listView1.Items[i].SubItems[2].Text);
                    xmlWriter.WriteEndElement();
                }

                xmlWriter.WriteEndElement();
                xmlWriter.WriteEndDocument();

                xmlWriter.Flush();
                xmlWriter.Close();
            }
        } 
        #endregion

        #region importToolStripMenuItem_Click
        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDlg = new OpenFileDialog();

            openFileDlg.Filter = "XML File (*.xml)|*.xml";

            if (openFileDlg.ShowDialog(this) == DialogResult.OK)
            {
                XmlReader xmlReader = XmlReader.Create(openFileDlg.OpenFile());

                while (xmlReader.Read())
                {
                    xmlReader.MoveToElement();

                    if (xmlReader.HasAttributes && xmlReader.Name.CompareTo("alarm") == 0)
                    {
                        string strTime = xmlReader.GetAttribute("time");
                        string strRepeats = xmlReader.GetAttribute("repeats");
                        string strDesc = xmlReader.GetAttribute("description");

                        this.listView1.Items.Add(new ListViewItem(new string[] { strTime, strRepeats, strDesc }));
                    }
                }

                xmlReader.Close();
            }
        } 
        #endregion

        private void preferencesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PrefFrm prefFrm = new PrefFrm();
            prefFrm.ShowDialog(this);
        }

        private void clearAlarmsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearAlarms();
        }

        private void editToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            EditSelectedAlarm();
        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            EditSelectedAlarm();
        }

        private void deleteToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            DeleteSelectedAlarm();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewAlarm();
        }


        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        
        private void checkForUpdatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            m_dlgUpdate.ShowDialog();
        }

        private void btnNewAlarm_Click(object sender, EventArgs e)
        {
            NewAlarm();
        }

        private void reportABugToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Srry, not implemented yet..");
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            DeleteSelectedAlarm();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            m_aboutFrm.ShowDialog();
        }
        #endregion


        #region DeleteSelectedAlarm
        private void DeleteSelectedAlarm()
        {
            if (this.listView1.SelectedIndices.Count > 0 && this.listView1.Items.Count > 0)
            {
                if (MessageBox.Show(this, "Are you sure?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    for (int i = 0; i < this.listView1.SelectedItems.Count; i++)
                        this.listView1.Items.Remove(this.listView1.SelectedItems[i]);
                }
            }
        }
        #endregion

        #region EditSelectedAlarm
        private void EditSelectedAlarm()
        {
            if (this.listView1.SelectedIndices.Count > 0 && this.listView1.Items.Count > 0)
            {
                int nListViewIndex = this.listView1.SelectedIndices[0];
                ListViewItem lviSelected = this.listView1.Items[nListViewIndex];

                DateTime dateTime = Convert.ToDateTime(lviSelected.SubItems[0].Text);
                string strRepeat = lviSelected.SubItems[1].Text;
                string strDescription = lviSelected.SubItems[2].Text;

                EditFrm editFrm = new EditFrm(dateTime, strRepeat, strDescription, nListViewIndex);
                editFrm.AlarmEdited += new EditFrm.EditAlarm(editFrm_AlarmEdited);
                editFrm.ShowDialog(this);
            }
        }
        #endregion

        #region AskIfClose
        private bool AskIfClose()
        {
            bool blnClose = false;
            if (MessageBox.Show(this, "Are you sure?", "Close",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                blnClose = true;
            }
            else
            {
                //Don't close
                blnClose = false;
            }

            return blnClose;
        }  
        #endregion

        #region NewAlarm
        private void NewAlarm()
        {
            CreateFrm CreateDlg = new CreateFrm();
            CreateDlg.NewAlarmAdded += new CreateFrm.AddNewAlarm(CreateDlg_NewAlarmAdded);
            CreateDlg.ShowDialog(this);
        }
        #endregion
    }
}
