﻿#region Header
/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#endregion

#region Using
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.Win32; 
#endregion

namespace Little_Alarm_Clock
{
    public partial class PrefFrm : Form
    {
        #region Ctor
        public PrefFrm()
        {
            InitializeComponent();

            if (Properties.Settings.Default.bPlaySound)
            {
                this.radioButtonSound.Checked = true;
                this.textBoxFile.Enabled = true;
                this.buttonSelectMusic.Enabled = true;
            }
            else
            {
                this.radioButtonQuiet.Checked = true;
                this.textBoxFile.Enabled = false;
                this.buttonSelectMusic.Enabled = false;
            }

            this.checkBoxStartUp.Checked = Properties.Settings.Default.bAutoStartup;
            this.checkBoxSysTray.Enabled = Properties.Settings.Default.bAutoStartup;
            if (this.checkBoxSysTray.Enabled)
                this.checkBoxSysTray.Checked = Properties.Settings.Default.bMinSysTray;

            this.textBoxFile.Text = Properties.Settings.Default.strAlarmPath;
            this.textBoxMsg.Font = Properties.Settings.Default.fontMsg;
            this.textBoxMsg.Text = string.Format("It is now {0}", DateTime.Now.ToString("hh:mm tt"));

            this.numUDSnooze.Value = Properties.Settings.Default.nSnoozeMinutes;
        } 
        #endregion

        #region AutoRun
        /// <summary>
        /// Adds little alarm clock to the startup folder
        /// </summary>
        private void AutoRun(bool bAdd, bool bMinimize)
        {
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", true);

            if (rk != null)
            {
                if (bAdd)
                {
                    if (bMinimize)
                        rk.SetValue(Application.ProductName, string.Format("\"{0}\" /startup", Application.ExecutablePath));
                    else
                        rk.SetValue(Application.ProductName, Application.ExecutablePath);
                }
                else
                    rk.DeleteValue(Application.ProductName, false);

                rk.Close();
            }
        } 
        #endregion

        #region buttonOK_Click
        private void buttonOK_Click(object sender, EventArgs e)
        {
            string strAlarmPath = string.Empty;
            bool blnOK = false;

            if (radioButtonBeep.Checked)
            {
                strAlarmPath = Application.StartupPath + "\\alarm.wav";
                blnOK = true;
            }
            else if (radioButtonSound.Checked)
            {
                strAlarmPath = this.textBoxFile.Text;

                blnOK = File.Exists(strAlarmPath);
                if (!blnOK)
                {
                    MessageBox.Show(this, "Please select an alarm sound to play", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    //File ok
                }
            }
            else
            {
                //Alarm disabled
                strAlarmPath = string.Empty;
                blnOK = true;
            }

            if (blnOK)
            {
                // Store settings
                Properties.Settings.Default.strAlarmPath = strAlarmPath;
                Properties.Settings.Default.fontMsg = this.textBoxMsg.Font;
                Properties.Settings.Default.bRepeatSound = checkBoxRepeat.Checked;
                Properties.Settings.Default.nSnoozeMinutes = (int)numUDSnooze.Value;
                Properties.Settings.Default.bPlaySound = radioButtonSound.Checked || radioButtonBeep.Checked;
                Properties.Settings.Default.bCheckUpdatesAtStart = checkBoxCheckUpdates.Checked;
                Properties.Settings.Default.bAutoStartup = checkBoxStartUp.Checked;
                if (checkBoxSysTray.Enabled)
                    Properties.Settings.Default.bMinSysTray = checkBoxSysTray.Checked;

                Properties.Settings.Default.Save();

                AutoRun(checkBoxStartUp.Checked, checkBoxSysTray.Checked);

                this.Close();
            }
            else
            {
                //Don't close
            }
        } 
        #endregion

        #region buttonFont_Click
        private void buttonFont_Click(object sender, EventArgs e)
        {
            this.fontDialog1.FontMustExist = true;
            this.fontDialog1.Font = this.textBoxMsg.Font;

            if (this.fontDialog1.ShowDialog(this) == DialogResult.OK)
            {
                Font fFont = fontDialog1.Font;

                Properties.Settings.Default.fontMsg = fFont;
                this.textBoxMsg.Font = fFont;
            }
            else
            {
                //cancel
            }
        } 
        #endregion

        #region buttonSelectMusic_Click
        private void buttonSelectMusic_Click(object sender, EventArgs e)
        {
            if (this.radioButtonSound.Checked)
            {
                OpenFileDialog openFileDlg = new OpenFileDialog();
                openFileDlg.Filter = "Audio Files(*.mp3;*.wav;*.wma;*.mid;*.midi)|*.mp3;*.wav;*.wma;*.mid;*.midi|All files (*.*)|*.*";

                if (openFileDlg.ShowDialog(this) == DialogResult.OK)
                    this.textBoxFile.Text = openFileDlg.FileName;
            }
        } 
        #endregion

        private void buttonDefault_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.fontMsg = new Font(FontFamily.GenericSansSerif, 8.25F);
            this.textBoxMsg.Font = Properties.Settings.Default.fontMsg;
        }

        private void checkBoxStartUp_CheckedChanged(object sender, EventArgs e)
        {
            this.checkBoxSysTray.Enabled = this.checkBoxStartUp.Checked;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void radioButtonSound_CheckedChanged(object sender, EventArgs e)
        {
            this.textBoxFile.Enabled = this.radioButtonSound.Checked;
            this.buttonSelectMusic.Enabled = this.radioButtonSound.Checked;
        }
    }
}
