﻿#region Header
/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#endregion

#region Using
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Little_Alarm_Clock.Logic; 
#endregion

namespace Little_Alarm_Clock
{
    public partial class UpdateFrm : Form
    {
        #region Member
        private ProgramUpdater m_puUpdater = null; 
        #endregion
        
        #region Ctor
        public UpdateFrm(ProgramUpdater puUpdater)
        {
            InitializeComponent();

            m_puUpdater = puUpdater;
        } 
        #endregion


        #region Events
        #region CheckForUpdate_Shown
        private void CheckForUpdate_Shown(object sender, EventArgs e)
        {
            labelCurrentVer.Text = m_puUpdater.CurrentVersion;
            labelLatestVer.Text = m_puUpdater.LatestVersion;

            bool blnDownloadEnabled = false;
            bool blnChangelogEnabled = false;
            Color clStatusColor = SystemColors.ControlText;
            string strStatusText = "(unknown)";

            switch (m_puUpdater.UpdateStatus)
            {
                case ProgramUpdateStatus.NewVersion:
                    {
                        blnDownloadEnabled = true;
                        blnChangelogEnabled = true;
                        clStatusColor = Color.LimeGreen;
                        strStatusText = "New version!";
                        break;
                    }
                case ProgramUpdateStatus.UpToDate:
                    {
                        blnDownloadEnabled = false;
                        blnChangelogEnabled = true;
                        clStatusColor = Color.Blue;
                        strStatusText = "Up to date";
                        break;
                    }
                case ProgramUpdateStatus.Error:
                case ProgramUpdateStatus.NoInit:
                default:
                    {
                        blnDownloadEnabled = false;
                        blnChangelogEnabled = false;
                        clStatusColor = Color.Red;
                        strStatusText = "Error!";

                        MessageBox.Show("Can't get update information",
                            "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        break;
                    }
            }

            buttonDownload.Enabled = blnDownloadEnabled;
            buttonChangelog.Enabled = blnChangelogEnabled;

            labelStatus.Text = strStatusText;
            labelStatus.ForeColor = clStatusColor;
        } 
        #endregion

        private void buttonDownload_Click(object sender, EventArgs e)
        {
            OpenURL(m_puUpdater.DownloadURL);
        } 

        private void buttonChangelog_Click(object sender, EventArgs e)
        {
            OpenURL(m_puUpdater.Changelog);
        }
        #endregion

        #region OpenURL
        private void OpenURL(string strUrl)
        {
            if (!string.IsNullOrEmpty(strUrl))
            {
                System.Diagnostics.Process.Start(strUrl);
            }
            else
            {
                //Don't open
            }
        } 
        #endregion

        private void UpdateFrm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
            {
                e.Cancel = true;
                this.Hide();
            }
            else
            {
                //OK
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Hide();
        }
    }
}
